
/*
	Password Prompter - PC Magazine password utility

	Copyright (c) 1999 Ziff-Davis Publishing Company.  All rights reserved.
	First published in PC Magazine, US Edition  June 8, 1999.

	Written by Steven E. Sipe


	This file implements the database class.
*/

#include "stdafx.h"

#include "passprom.h"
#include "database.h"

// 
// Database Header
//

// Constructor
CPPDBHeader::CPPDBHeader()
{
	m_nEntries = 0;
	m_nKeySize = KEY_SIZE;
	m_sFileVersion = 0x0100;
	m_abyPassword.byLen = 0;
	m_abyValid.byLen = strlen(VALID);
	memcpy(m_abyValid.abyData,VALID,strlen(VALID));
}

// Destructor
CPPDBHeader::~CPPDBHeader()
{
	ClearIndex();
}

// Removes entries from the hash-key table
void CPPDBHeader::ClearIndex()
{
	POSITION pos = m_mapIndex.GetStartPosition();

	// Iterate through each entry
	for(int i = 0; i < m_mapIndex.GetCount(); i++)
	{
		CPPDBEntry *pEntry;
		CString strKey;

		// Get it's associated object and delete it
		m_mapIndex.GetNextAssoc(pos,strKey,(CObject *&) pEntry);
		delete pEntry;
	}

	// Remove the key values too
	m_mapIndex.RemoveAll();
}


// Handles reading and writing header data and entries from/to 
// secondary storage
void CPPDBHeader::Serialize(CArchive &ar)
{
	CMangle Mangle;

	if(ar.IsStoring())
	{
		BYTE_ARRAY abyTempKey, abyTempValid;

		// Generate a random cipher key
		m_abyKey.byLen = KEY_SIZE;
		Mangle.GenKey(m_abyKey.abyData,m_abyKey.byLen);

		// Mangle the cipher key using the password
		memcpy(abyTempKey.abyData,m_abyKey.abyData,m_abyKey.byLen);
		abyTempKey.byLen = m_abyKey.byLen;

		if(m_abyPassword.byLen > 0)
		{
			Mangle.SetKey(m_abyPassword.abyData,m_abyPassword.byLen);
			Mangle.Encode(abyTempKey.abyData,abyTempKey.byLen);
		}

		Mangle.SetKey(abyTempKey.abyData,abyTempKey.byLen);

		// Write the header
		ar.Write(&m_sFileVersion,sizeof(m_sFileVersion));
		ar.Write(&m_nEntries,sizeof(m_nEntries));
		ar.Write(&m_nKeySize,sizeof(m_nKeySize));
		ar.Write(&abyTempKey.byLen,sizeof(abyTempKey.byLen));
		ar.Write(abyTempKey.abyData,abyTempKey.byLen);

		// Write and encode the validity check entry
		abyTempValid.byLen = strlen(VALID);
		memcpy(abyTempValid.abyData,VALID,abyTempValid.byLen);
		Mangle.SetKey(m_abyKey.abyData,m_abyKey.byLen);
		Mangle.Encode(abyTempValid.abyData,abyTempValid.byLen);
		ar.Write(&abyTempValid.byLen,sizeof(abyTempValid.byLen));
		ar.Write(abyTempValid.abyData,abyTempValid.byLen);
	}
	else
	{
		// Read the header
		ar.Read(&m_sFileVersion,sizeof(m_sFileVersion));
		ar.Read(&m_nEntries,sizeof(m_nEntries));
		ar.Read(&m_nKeySize,sizeof(m_nKeySize));
		ar.Read(&m_abyKey.byLen,sizeof(m_abyKey.byLen));
		ar.Read(m_abyKey.abyData,m_abyKey.byLen);

		ar.Read(&m_abyValid.byLen,sizeof(m_abyValid.byLen));
		ar.Read(m_abyValid.abyData,m_abyValid.byLen);

		// De-mangle the key using the password
		if(m_abyPassword.byLen > 0)
		{
			Mangle.SetKey(m_abyPassword.abyData,m_abyPassword.byLen);
			Mangle.Decode(m_abyKey.abyData,m_abyKey.byLen);
		}

		// De-mangle the validity check entry
		Mangle.SetKey(m_abyKey.abyData,m_abyKey.byLen);
		Mangle.Decode(m_abyValid.abyData,m_abyValid.byLen);

		// Initialize the index map
		ClearIndex();
	}

	// Handle the individual entries
	POSITION pos = m_mapIndex.GetStartPosition();

	for(int i = 0; i < m_nEntries; i++)
	{
		CPPDBEntry *pEntry;
		CString strKey;

		if(ar.IsStoring())
			m_mapIndex.GetNextAssoc(pos,strKey,(CObject *&) pEntry);
		else pEntry = new CPPDBEntry;

		pEntry->SetKey(m_abyKey.abyData,m_abyKey.byLen);
		pEntry->Serialize(ar);

		if(ar.IsLoading())
		{
			CString strKey = (char *) pEntry->m_abyDescription.abyData;

			strKey.MakeLower();
			m_mapIndex.SetAt(strKey,(CObject *&) pEntry);
		}
	}
}

// Returns the current password
LPCTSTR CPPDBHeader::GetPassword(void) 
{ 
	static char szTemp[200];

	memcpy(szTemp,m_abyPassword.abyData,m_abyPassword.byLen);
	szTemp[m_abyPassword.byLen] = '\0';

	return(szTemp);
}

// Sets the new password
void CPPDBHeader::SetPassword(LPCTSTR lpszPassword, BOOL bSave) 
{ 
	memcpy(m_abyPassword.abyData,lpszPassword,strlen(lpszPassword));
	m_abyPassword.byLen = strlen(lpszPassword);

	if(bSave) Save();
}

// Loads entries from the database into memory
BOOL CPPDBHeader::Load()
{
	CFile file;

	try
	{
		// Does the file exist?
		if(file.Open(m_strFileName,CFile::modeRead))
		{
			// Make archive stuff local to this block so the archive writes
			// are flushed before the file is closed below
			{
				CArchive ar(&file,CArchive::load);
				Serialize(ar);
			}

			file.Close();
		}
	}
	catch(CFileException *e)
	{
		if(e->m_cause != CFileException::fileNotFound)
			return(FALSE);
	}
	catch(...)
	{
		return(FALSE);
	}

	return(TRUE);
}

// Saves database entries to the database file
BOOL CPPDBHeader::Save()
{
	CFile file;

	try
	{
		if(file.Open(m_strFileName,CFile::modeCreate|CFile::modeWrite))
		{
			// Make archive stuff local to this block so the archive writes
			// are flushed before the file is closed below
			{
				CArchive ar(&file,CArchive::store);
				Serialize(ar);
			}

			file.Close();
		}
	}
	catch(...)
	{
		return(FALSE);
	}

	return(TRUE);
}

// Locates the specified entry using its string value
CPPDBEntry *CPPDBHeader::Find(LPCTSTR lpszDescription)
{
	CString strKey = lpszDescription;
	CPPDBEntry *pEntry;

	// Always search in lowercase -- use CMap lookup routine
	strKey.MakeLower();
	BOOL bRet = m_mapIndex.Lookup(strKey,(CObject *&) pEntry);

	return(bRet?pEntry:NULL);
}

// Sets the value and length of the specified data field
void CPPDBHeader::SetField(BYTE_ARRAY& To, LPCTSTR lpszFrom)
{
	strcpy((char *) To.abyData,lpszFrom);
	To.byLen = strlen(lpszFrom);
}

// Adds an entry to the database
BOOL CPPDBHeader::Add(LPCTSTR lpszDescription, LPCTSTR lpszUserID, 
				LPCTSTR lpszPassword, LPCTSTR lpszHint, LPCTSTR lpszComment,
				LPCTSTR lpszURL)
{
	// Make sure this entry doesn't already exist
	if(!Find(lpszDescription))
	{
		CString strKey = lpszDescription;
		CPPDBEntry *pEntry = new CPPDBEntry;

		strKey.MakeLower();

		// Set the entry fields
		SetField(pEntry->m_abyDescription,lpszDescription);
		SetField(pEntry->m_abyUserID,lpszUserID);
		SetField(pEntry->m_abyPassword,lpszPassword);
		SetField(pEntry->m_abyHint,lpszHint);
		SetField(pEntry->m_abyComment,lpszComment);
		SetField(pEntry->m_abyURL,lpszURL);

		// Save the key
		m_mapIndex.SetAt(strKey,pEntry);
		m_nEntries++;
	}
	else
	{
		AfxMessageBox(IDS_ALREADY_EXISTS,MB_ICONWARNING|MB_OK);
		return(FALSE);
	}

	return(TRUE);
}

// Modifies the specifed entry
BOOL CPPDBHeader::Modify(LPCTSTR lpszKey, LPCTSTR lpszDescription, 
				LPCTSTR lpszUserID, LPCTSTR lpszPassword, LPCTSTR lpszHint, 
				LPCTSTR lpszComment,LPCTSTR lpszURL)
{
	CPPDBEntry *pEntry;

	// Will this produce a duplicate entry?
	if(Find(lpszDescription) && strcmp(lpszKey,lpszDescription))
	{
		// Yep...get out
		AfxMessageBox(IDS_ALREADY_EXISTS,MB_ICONWARNING|MB_OK);
		return(FALSE);
	}

	// Find the original entry
	pEntry = Find(lpszKey);
	if(pEntry)
	{
		// Update the new information
		SetField(pEntry->m_abyDescription,lpszDescription);
		SetField(pEntry->m_abyUserID,lpszUserID);
		SetField(pEntry->m_abyPassword,lpszPassword);
		SetField(pEntry->m_abyHint,lpszHint);
		SetField(pEntry->m_abyComment,lpszComment);
		SetField(pEntry->m_abyURL,lpszURL);

		CString strNewKey = lpszDescription;
		CString strOldKey = lpszKey;

		strNewKey.MakeLower();
		strOldKey.MakeLower();

		// Save the new key
		m_mapIndex.RemoveKey(strOldKey);
		m_mapIndex.SetAt(strNewKey,pEntry);
	}

	return(TRUE);
}

// Removes the specified entry
BOOL CPPDBHeader::Remove(LPCTSTR lpszDescription)
{
	CPPDBEntry *pEntry;

	// Locate the entry
	pEntry = Find(lpszDescription);
	if(pEntry)
	{
		CString strKey = lpszDescription;

		strKey.MakeLower();

		// Delete the entry and remove its key
		delete pEntry;
		m_mapIndex.RemoveKey(strKey);

		m_nEntries--;
	}
	else return(FALSE);

	return(TRUE);
}

// Loads the selection list combobox with database entries
void CPPDBHeader::FillList(CComboBox& combo)
{
	POSITION pos = m_mapIndex.GetStartPosition();

	combo.ResetContent();

	// Iterate through all entries
	for(int i = 0; i < m_mapIndex.GetCount(); i++)
	{
		CPPDBEntry *pEntry;
		CString strKey;

		m_mapIndex.GetNextAssoc(pos,strKey,(CObject *&) pEntry);

		// Add the description
		combo.AddString((char *) pEntry->m_abyDescription.abyData);
	}
}

// 
// Database Records
//

// Constructor
CPPDBEntry::CPPDBEntry()
{
	Init();
}

// Destructor
CPPDBEntry::~CPPDBEntry()
{
}

// Clear each of the entry's fields
void CPPDBEntry::Init()
{
	memset(&m_abyDescription,0,sizeof(m_abyDescription));
	memset(&m_abyUserID,0,sizeof(m_abyUserID));
	memset(&m_abyPassword,0,sizeof(m_abyPassword));
	memset(&m_abyHint,0,sizeof(m_abyHint));
	memset(&m_abyComment,0,sizeof(m_abyComment));
	memset(&m_abyURL,0,sizeof(m_abyURL));
}

// Gets a field from the archive
void CPPDBEntry::GetField(BYTE_ARRAY& Data, CArchive& ar)
{
	ar.Read(&Data.byLen,sizeof(Data.byLen));
	ar.Read(Data.abyData,Data.byLen);

	// De-managle the data
	m_Mangle.Decode(Data.abyData,Data.byLen);
}

// Writes a field to the archive
void CPPDBEntry::PutField(BYTE_ARRAY& Data, CArchive& ar)
{
	BYTE byTemp[500];

	// Mangle the data
	memcpy(byTemp,Data.abyData,Data.byLen);
	m_Mangle.Encode(byTemp,Data.byLen);

	ar.Write(&Data.byLen,sizeof(Data.byLen));
	ar.Write(byTemp,Data.byLen);
}

// Handles reading and writing of entries from/to database records
void CPPDBEntry::Serialize(CArchive &ar)
{
	if(ar.IsStoring())
	{
		// Write the entry
		PutField(m_abyDescription,ar);
		PutField(m_abyUserID,ar);
		PutField(m_abyPassword,ar);
		PutField(m_abyHint,ar);
		PutField(m_abyComment,ar);
		PutField(m_abyURL,ar);
	}
	else
	{
		// Read the entry
		GetField(m_abyDescription,ar);
		GetField(m_abyUserID,ar);
		GetField(m_abyPassword,ar);
		GetField(m_abyHint,ar);
		GetField(m_abyComment,ar);
		GetField(m_abyURL,ar);
	}
}

